#------------------------------Dissagregation
from __future__ import print_function, division
import time
import tables
from matplotlib import rcParams
import matplotlib.pyplot as plt
import pandas as pd
import numpy as np
from six import iteritems

from nilmtk import DataSet, TimeFrame, MeterGroup, HDFDataStore
from nilmtk.legacy.disaggregate import CombinatorialOptimisation, FHMM, Hart85, MLE
import nilmtk.utils


def predict(clf, test_elec, sample_period, timezone):
    pred = {}
    gt= {}
    
    # "ac_type" varies according to the dataset used. 
    # Make sure to use the correct ac_type before using the default parameters in this code.    
    for i, chunk in enumerate(test_elec.mains().load(physical_quantity = 'power', ac_type = 'active', sample_period=sample_period)):
        chunk_drop_na = chunk.dropna()
        pred[i] = clf.disaggregate_chunk(chunk_drop_na)
        gt[i]={}

        for meter in test_elec.submeters().meters:
            # Only use the meters that we trained on (this saves time!)    
            gt[i][meter] = next(meter.load(physical_quantity = 'power', ac_type = 'active', sample_period=sample_period))
        gt[i] = pd.DataFrame({k:v.squeeze() for k,v in iteritems(gt[i]) if len(v)}, index=next(iter(gt[i].values())).index).dropna()
        
    # If everything can fit in memory
    gt_overall = pd.concat(gt)
    gt_overall.index = gt_overall.index.droplevel()
    pred_overall = pd.concat(pred)
    pred_overall.index = pred_overall.index.droplevel()

    # Having the same order of columns
    gt_overall = gt_overall[pred_overall.columns]
    
    #Intersection of index
    gt_index_utc = gt_overall.index.tz_convert("UTC")
    pred_index_utc = pred_overall.index.tz_convert("UTC")
    common_index_utc = gt_index_utc.intersection(pred_index_utc)
    
    common_index_local = common_index_utc.tz_convert(timezone)
    gt_overall = gt_overall.loc[common_index_local]
    pred_overall = pred_overall.loc[common_index_local]
    appliance_labels = [m for m in gt_overall.columns.values]
    gt_overall.columns = appliance_labels
    pred_overall.columns = appliance_labels
    return gt_overall, pred_overall


rcParams['figure.figsize'] = (6, 3)

train = DataSet('nilm_5679_1min_net.h5')
test = DataSet('nilm_5679_1min_net.h5')
building = 5679

train.set_window(start="2019-08-01",end='2019-08-26')
test.set_window(start="2019-08-26",end='2019-08-30')

train_elec = train.buildings[5679].elec
test_elec = test.buildings[5679].elec



classifiers = {'CO':CombinatorialOptimisation(),'FHMM':FHMM()}#,'Hart':Hart85()}
predictions = {}
sample_period = 180
end=time.time()
for clf_name, clf in classifiers.items():
    print("*"*20)
    print(clf_name)
    print("*" *20)
    start = time.time()
    print("Testime(OC) 1st(x) 2nd(O)=", end-start, "seconds------------",start,end)

    clf.train(train_elec, sample_period=sample_period)
    end = time.time()
    print("Runtime =", end-start, "seconds.------------",start,end)
    gt, predictions[clf_name] = predict(clf, test_elec, sample_period, train.metadata['timezone'])

testtime = time.time()
print("Testime(FHMM)=", testtime-end, "seconds------------",end,testtime)
tables.file._open_files.close_all()

print("Training is finisheddddd")

appliance_labels = [m.label() for m in gt.columns.values]

gt.columns = appliance_labels
predictions['CO'].columns = appliance_labels
predictions['FHMM'].columns = appliance_labels
 #predictions['Hart'].columns = appliance_labels

#15min_sample_period=60
predictions['CO']['Electric vehicle'].plot(label="Predicted EV (CO)")
gt['Electric vehicle'].plot(label="Ground Truth", alpha=0.6)#, linewidth=0.8)
plt.legend(loc='upper center', bbox_to_anchor=(0.5,1.18),prop={'size':10}, ncol=2)

plt.savefig('1min_oc_long_net.png')
plt.show()

#15min_sample_period=60
predictions['CO']['Electric vehicle'].plot(label="Predicted EV (FHMM)")
gt['Electric vehicle'].plot(label="Ground Truth", alpha=0.6)#, linewidth=0.8)
plt.legend(loc='upper center', bbox_to_anchor=(0.5,1.18),prop={'size':10}, ncol=2)

plt.savefig('1min_fhmm_long_net.png')
plt.show()

#15min

print('the endddddd')

rmse = {}
for clf_name in classifiers.keys():
    rmse[clf_name] = nilmtk.utils.compute_rmse(gt, predictions[clf_name])

rmse = pd.DataFrame(rmse)

print(rmse)




