import os
import re
# import datetime
import sys
from os.path import join, isdir, isfile, dirname, abspath
import pandas as pd
import yaml
import psycopg2 as db
from nilmtk.measurement import measurement_columns
from nilmtk.measurement import LEVEL_NAMES
from nilmtk.datastore import Key
from nilm_metadata import convert_yaml_to_hdf5
from nilmtk.utils import get_module_directory
import shutil
import tempfile
from datetime import datetime
import numpy as np

feed_mapping = {
                'air1': {'type': 'air conditioner'},
                'air2': {'type': 'air conditioner'},
                'air3': {'type': 'air conditioner'},
                'airwindowunit1': {'type': 'air conditioner'},
                'aquarium1': {'type': 'appliance'},
                'bathroom1': {'type': 'sockets', 'room': 'bathroom'},
                'bathroom2': {'type': 'sockets', 'room': 'bathroom'},
                'bedroom1': {'type': 'sockets', 'room': 'bedroom'},
                'bedroom2': {'type': 'sockets', 'room': 'bedroom'},
                'bedroom3': {'type': 'sockets', 'room': 'bedroom'},
                'bedroom4': {'type': 'sockets', 'room': 'bedroom'},
                'bedroom5': {'type': 'sockets', 'room': 'bedroom'},
                'battery1': {'type': 'battery'},                       #new field, need mapping
                'car1': {'type': 'electric vehicle'},
                'car2': {'type': 'electric vehicle'},
                'circpump1': {'type': 'circpump'},                      #new field, need mapping
                'clotheswasher1': {'type': 'washing machine'},
                'clotheswasher_dryg1': {'type': 'washer dryer'},
                'diningroom1': {'type': 'sockets', 'room': 'dining room'},
                'diningroom2': {'type': 'sockets', 'room': 'dining room'},
                'dishwasher1': {'type': 'dish washer'},
                'disposal1': {'type': 'waste disposal unit'},
                'drye1': {'type': 'spin dryer'},
                'dryg1': {'type': 'spin dryer'},
                'freezer1': {'type': 'freezer'},
                'furnace1': {'type': 'electric furnace'},
                'furnace2': {'type': 'electric furnace'},
                'garage1': {'type': 'sockets', 'room': 'dining room'},
                'garage2': {'type': 'sockets', 'room': 'dining room'},
                'grid': {'type': 'grid'},
                'heater1': {'type': 'electric space heater'},
                'heater2': {'type': 'electric space heater'},
                'heater3': {'type': 'electric space heater'},
                'housefan1': {'type': 'electric space heater'},
                'icemaker1': {'type': 'appliance'},
                'jacuzzi1': {'type': 'electric hot tub heater'},
                'kitchen1': {'type': 'sockets', 'room': 'kitchen'},
                'kitchen2': {'type': 'sockets', 'room': 'kitchen'},
                'kitchenapp1': {'type': 'sockets', 'room': 'kitchen'},
                'kitchenapp2': {'type': 'sockets', 'room': 'kitchen'},
                'lights_plugs1': {'type': 'light'},
                'lights_plugs2': {'type': 'light'},
                'lights_plugs3': {'type': 'light'},
                'lights_plugs4': {'type': 'light'},
                'lights_plugs5': {'type': 'light'},
                'lights_plugs6': {'type': 'light'},
                'livingroom1': {'type': 'sockets', 'room': 'living room'},
                'livingroom2': {'type': 'sockets', 'room': 'living room'},
                'microwave1': {'type': 'microwave'},
                'office1': {'type': 'sockets', 'room': 'office'},
                'outsidelights_plugs1': {'type': 'sockets', 'room': 'outside'},
                'outsidelights_plugs2': {'type': 'sockets', 'room': 'outside'},
                'oven1': {'type': 'oven'},
                'oven2': {'type': 'oven'},
                'pool1': {'type': 'electric swimming pool heater'},
                'pool2': {'type': 'electric swimming pool heater'},
                'poollight1': {'type': 'light'},
                'poolpump1': {'type': 'electric swimming pool heater'},
                'pump1': {'type': 'appliance'},
                'range1': {'type': 'stove'},
                'refrigerator1': {'type': 'fridge'},
                'refrigerator2': {'type': 'fridge'},
                'security1': {'type': 'security alarm'},
                'sewerpump1': {'type': 'sewerpump'},               #new field, need mapping
                'shed1': {'type': 'sockets', 'room': 'shed'},
                'solar': {'type': 'solar'},
                'solar2': {'type': 'solar'},
                'sprinkler1': {'type': 'appliance'},
                'sumppump1': {'type': 'sumppump'},                #new field, need mapping
                'utilityroom1': {'type': 'sockets', 'room': 'utility room'},
                'venthood1': {'type': 'appliance'},
                'waterheater1': {'type': 'electric water heating appliance'},
                'waterheater2': {'type': 'electric water heating appliance'},
                'wellpump1': {'type': 'wellpump'},                #new field, need mapping
                'winecooler1': {'type': 'appliance'},
                'leg1v':{},
                'leg2v':{}
                }

feed_ignore = ['solar', 'solar2', 'grid','leg1v', 'leg2v', 'battery1', 'circpump1','sewerpump1', 'sumppump1', 'wellpump1']



def _dataport_dataframe_to_hdf(dataport_dataframe,
                               store,metadata_dir,
                               nilmtk_building_id=5679,
                               dataport_building_id=5679,
                               timestamp_name='localminute',
                               user_selected_table='eg_realpower_1s'):
    
    local_dataframe = dataport_dataframe.copy()
#     date_rng = pd.date_range(start='8/1/2019', end='8/31/2019', freq='S')
#     date_rng = pd.to_datetime(local_dataframe[timestamp_name], infer_datetime_format=True)
#     local_dataframe = pd.DataFrame(date_rng, columns=[timestamp_name])
     # set timestamp as frame index
    local_dataframe = local_dataframe.set_index(timestamp_name)
    print(local_dataframe.head())
    # set timezone
#     local_dataframe = local_dataframe.tz_localize('US/Eastern')#('US/Central')

    # remove timezone information to avoid append errors
#     local_dataframe[timestamp_name] = pd.DatetimeIndex([i.replace(tzinfo='US/Eastern') for i in local_dataframe[timestamp_name]])
    # set timestamp as frame index
#     local_dataframe = local_dataframe.set_index(timestamp_name)

    # set timezone
#     local_dataframe.index = pd.DatetimeIndex(local_dataframe.index).tz_convert('US/Eastern')#('US/Central')
    
    print('\n --------')
    
    # remove timestamp column from dataframe
    feeds_dataframe = local_dataframe.drop('dataid', axis=1)
    print(feeds_dataframe.head())
    # Column names for dataframe
    column_names = [('power','active')]
    # convert from kW to W for realpower data
    if(user_selected_table.find('_realpower_') is not None):
        feeds_dataframe = feeds_dataframe.mul(1000)
    # building metadata
    building_metadata = {}
    building_metadata['instance'] = nilmtk_building_id
    building_metadata['original_name'] = int(dataport_building_id)  # use python int
    building_metadata['elec_meters'] = {}
    building_metadata['appliances'] = []

    # initialise dict of instances of each appliance type
    instance_counter = {}

    meter_id = 1
    for column in feeds_dataframe.columns:
        print(column)
        if feeds_dataframe[column].notnull().sum() > 0 and not column in feed_ignore:

            # convert timeseries into dataframe
            feed_dataframe = pd.DataFrame(feeds_dataframe[column])

            # set column names
            feed_dataframe.columns = pd.MultiIndex.from_tuples(column_names)

            # Modify the column labels to reflect the power measurements recorded.
            feed_dataframe.columns.set_names(LEVEL_NAMES, inplace=True)
            
            key = Key(building=nilmtk_building_id, meter=meter_id)
            

            # store dataframe
            store.put(str(key), feed_dataframe, format='table', append=True)
            store.flush()

            # elec_meter metadata
            if column == ('power_consumed'):#'power_log' or 'power_consumed' or
                meter_metadata = {'device_model': 'eGauge','site_meter': True}
            else:
                meter_metadata = {'device_model': 'eGauge', 'submeter_of': 0}
            building_metadata['elec_meters'][meter_id] = meter_metadata
            # appliance metadata
            if column != ('power_consumed'):#'power_log' and 'power_consumed' and
                # original name and meter id
                appliance_metadata = {'original_name': column,'meters': [meter_id]}
                # appliance type and room if available
                appliance_metadata.update(feed_mapping[column])
                # appliance instance number
                if instance_counter.get(appliance_metadata['type']) == None:
                    instance_counter[appliance_metadata['type']] = 0
                instance_counter[appliance_metadata['type']] += 1
                appliance_metadata['instance'] = instance_counter[appliance_metadata['type']]
                building_metadata['appliances'].append(appliance_metadata)

            meter_id += 1

    # write building yaml to file
    building = 'building{:d}'.format(nilmtk_building_id)
    yaml_full_filename = join(metadata_dir, building + '.yaml')
    with open(yaml_full_filename, 'w') as outfile:
        outfile.write(yaml.dump(building_metadata))

    return 0

fname='ny2_id5679.csv'
df = pd.read_csv(fname)
print(df.shape)
df['localminute'] = pd.to_datetime(df.localminute)
df_sorted = df.sort_values(by='localminute',ascending=True)
# df_sorted=df_sorted.assign(power_log = df_sorted["grid"] - df_sorted["solar"])
df_sorted=df_sorted.assign(power_consumed = df_sorted["air1"] +df_sorted["car1"]+df_sorted["furnace1"]\
                           +df_sorted["garage1"]+df_sorted["kitchenapp1"]+df_sorted["kitchenapp2"])
# df_sorted=df_sorted.assign(net_power = df_sorted["grid"] - df_sorted["power_consumed"]+df_sorted["solar"])

hdf_filename = 'nilm_5679_1s.h5'

if os.path.exists(hdf_filename):
    os.remove(hdf_filename)

store = pd.HDFStore(hdf_filename, 'a', complevel=9, complib='zlib')

 # Create a temporary metadata dir, remove existing building
    # yaml files in module dir (if any)
original_metadata_dir = join(get_module_directory(),'dataset_converters','dataport','metadata')
tmp_dir = tempfile.mkdtemp()
metadata_dir = join(tmp_dir, 'metadata')
shutil.copytree(original_metadata_dir, metadata_dir)
print("Using temporary dir for metadata:", metadata_dir)

for f in os.listdir(metadata_dir):
    if re.search('^building', f):
        os.remove(join(metadata_dir, f))

_dataport_dataframe_to_hdf(df_sorted, store,metadata_dir)

store.close()

# write yaml to hdf5
# dataset.yaml and meter_devices.yaml are static, building<x>.yaml are dynamic
convert_yaml_to_hdf5(metadata_dir, hdf_filename)

# remote the temporary dir when finished
shutil.rmtree(tmp_dir)



